//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Group/ItemCatalog.h
//! @brief     Defines class ItemCatalog
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_GROUP_ITEMCATALOG_H
#define BORNAGAIN_GUI_MODEL_GROUP_ITEMCATALOG_H

#include "Base/Utils/IFactory.h"
#include <QMap>
#include <QStringList>
#include <memory>

class SessionItem;

//! Catalog of SessionItem%s. A single instance is created and used in ItemFactory.cpp.

class ItemCatalog {
public:
    struct UiInfo {
        QString menuEntry;
        QString description;
        QString iconPath;
    };

    ItemCatalog();

    static ItemCatalog& instance();

    std::unique_ptr<SessionItem> createItemPtr(const QString& modelType) const;

    static QStringList validTopItemTypes();
    static QStringList formFactorHardParticleTypes();
    static QStringList formFactorRippleTypes();
    static QStringList formFactorTypes();
    static QStringList interferenceFunctionTypes();
    static QStringList particleAssemblyTypes();

    static bool isFormFactorModelType(const QString& modeltype);

    template <typename ItemClass> UiInfo uiInfo() const { return uiInfo(ItemClass::M_TYPE); }

    UiInfo uiInfo(const QString& modelType) const;

    QList<UiInfo> uiInfos(const QStringList& modelTypes) const;

private:
    template <typename T> void addItem();
    template <typename T>
    void addItem(const QString& menuEntry, const QString& iconPath = "",
                 const QString& description = "");

    IFactory<QString, SessionItem> m_data;
    QMap<QString, UiInfo> m_uiInfos;
};


#endif // BORNAGAIN_GUI_MODEL_GROUP_ITEMCATALOG_H
