//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Types/Unit.cpp
//! @brief     Implements class Unit
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Types/Unit.h"
#include "Base/Const/Units.h"
#include "Base/Utils/Assert.h"
#include <QString>

double convert(double d, Unit from, Unit to)
{
    if (from == to)
        return d;

    if (from == Unit::angstrom && to == Unit::nanometer)
        return d / 10.0;

    if (from == Unit::nanometer && to == Unit::angstrom)
        return d * 10.0;

    if (from == Unit::angstromPower2 && to == Unit::nanometerPower2)
        return d / 100.0;

    if (from == Unit::nanometerPower2 && to == Unit::angstromPower2)
        return d * 100.0;

    if (from == Unit::angstromPowerMinus2 && to == Unit::nanometerPowerMinus2)
        return d * 100.0;

    if (from == Unit::nanometerPowerMinus2 && to == Unit::angstromPowerMinus2)
        return d / 100.0;

    if (from == Unit::radiant && to == Unit::degree)
        return Units::rad2deg(d);

    if (from == Unit::degree && to == Unit::radiant)
        return Units::deg2rad(d);

    if (from == Unit::other || to == Unit::other) {
        ASSERT(false); // no conversion possible
        return d;
    }

    ASSERT(false); // no conversion implemented
    return d;
}

QString unitAsString(const Unit& unit)
{
    switch (unit) {
    case Unit::unitless:
        return "";
    case Unit::nanometer:
        return "nm";
    case Unit::nanometerPower2:
        return u8"nm\u00B2";
    case Unit::nanometerPowerMinus2:
        return u8"1/nm\u00B2";
    case Unit::angstrom:
        return u8"\u00c5";
    case Unit::angstromPower2:
        return u8"\u00c5\u00B2";
    case Unit::angstromPowerMinus2:
        return u8"1/\u00c5\u00B2";
    case Unit::degree:
        return "°";
    case Unit::radiant:
        return "rad";
    case Unit::other:
        ASSERT(false); // this function should not be called for Unit::other
        return "";
    default:
        ASSERT(false); // no string implemented
        return "";
    }
}
